package org.ids_mannheim;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

public class Progressbar {
    private static final int width = 40;
    //    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("ccc dd. LLL HH:mm:ss");
    private final long max;
    private final long start;
    private long current;
    private long lastUpdate;

    public Progressbar(long max, String name) {
        this.start = System.currentTimeMillis();
        this.max = max;
        System.err.println(name + ":");
        this.printBar(false);
    }

    public synchronized void update(long i) {
        this.current += i;
        if ((System.currentTimeMillis() - this.lastUpdate) > 100) {
            this.lastUpdate = System.currentTimeMillis();
            this.printBar(false);
        }
    }

    public void finish() {
        this.current = this.max;
        this.printBar(true);
    }

    public String shortDateTimeFromMillis(long millis) {
        LocalDateTime date = LocalDateTime.ofInstant(Instant.ofEpochMilli(millis), ZoneId.systemDefault());
        return date.format(formatter);
    }

    public String hmsFromMillis(long millis) {
        int seconds = (int) (millis / 1000) % 60;
        int minutes = (int) (millis / 1000) % 3600 / 60;
        int hours = (int) (millis / 1000) / 3600;
        return String.format("%02d:%02d:%02d", hours, minutes, seconds);
    }

    private void printBar(boolean finished) {
        double numbar = Math.floor(width * (double) current / (double) max);
        StringBuilder strbar = new StringBuilder();
        int i;
        for (i = 0; i < numbar - 1; i++) {
            strbar.append("=");
        }
        for (i = (int) numbar - 1; i < numbar; i++) {
            if (i < width - 1) {
                strbar.append(">");
            } else {
                strbar.append("=");
            }
        }
        for (i = (int) numbar; i < width; i++) {
            strbar.append(" ");
        }
        long elapsed = (System.currentTimeMillis() - this.start);
        long ttg = (long) (((double) elapsed / (double) current) * (max - current));
        System.err.print(String.format("\r%3d%% [%s] %.1fMB/s ELA: %s TTG: %s ETA: %s\u001B[0K",
                (int) (100 * (double) current / (double) max),
                strbar,
                1000.0 * current / (1024 * 1024) / elapsed,
                hmsFromMillis(elapsed),
                hmsFromMillis(ttg),
                shortDateTimeFromMillis(start + elapsed + ttg)
                ));
        if (finished) {
            System.err.print("\n");
        }
        System.err.flush();
    }
}